//------------------------------------------------|
// C Header File																	|
// Created 11/29/00; 5:26:43 PM										|
//------------------------------------------------|
//Let me explain breifly how the enemies work:    |
//																								|
//Each enemy has ten attributes, and all are      |
//listed in a gigantic array, called enemy.       |
//Each level can have 30 enemies, for a total of  |
//90 enemies and 900 attributes. A list of the    |
//attributes:                                     |
//		1) type (1 = one dir., 2 = two dir. 3=fly)  |
//		2) starting x coord. (where enemy starts)   |
//		3) starting y coord.												|
//		4) end x coord.	(used for two dir. and fly) |
//		5) end y coord. (used for fly)							|
//		6) current x coord. (where enemy is now)    |
//    7) current y coord.													|
//		8) horizontal direction (1=right, 2=left)   |
//		9) vertical direction (1=down, 2=up)				|
//   10) screen enemy is on												|
//------------------------------------------------|
//All the enemy function use these attributes to  |
//calculate where the enemy is, when it should    |
//bounce, when it hits things, etc.               |
//------------------------------------------------|
//All code was done by Sean Kelly, www.skti.org   |
//Feel free to use it, but give me some credit    |
//------------------------------------------------|


//------------------------------------------
//collision detection horizontally for enemy
//------------------------------------------
int checkHorz(int theX,int theY,int theDir,int w)
{
	int ex=1;
	if(theDir==2)
		theDir=-1;
	if(theDir==1)
		ex=8;
	if(map[theY/8][(theX+theDir*ex)/8].type<3) return theDir;
	if(theDir==-1)
		enemy[w+7].type=1;
	else
		enemy[w+7].type=2;
	return theDir*-1;
}

//--------------------------------------------------------
//checks below enemy to make sure they are on solid ground
//--------------------------------------------------------
int checkEDown(int theX,int theY)
{
	if(map[(theY+8)/8][theX/8].type>2) return 0;
	if(map[(theY+8)/8][(theX+7)/8].type>2) return 0;
	return 1;
}

//---------------------------------------------------------------
//checks above enemy to make sure they are not below solid ground
//---------------------------------------------------------------
int checkEUp(int theX,int theY)
{
	if(theY==18) return 0;
	if(map[(theY-1)/8][theX/8].type>2) return 0;
	if(map[(theY-1)/8][(theX+7)/8].type>2) return 0;
	return 1;
}

//------------------------------------------
//if enemy goes off screen, stop handling it
//------------------------------------------
void borderCheck(int theX,int theY,int w)
{
  int yes=0;
	if(theX>=160) yes=1;
	if(theX==0)
		enemy[w+7].type=1;
	if(theY>=75) yes=1;
	if(yes)
	 {
	 	enemy[w].type=0;
	 	masker(theX,theY+add);
	 }
}

//--------------------------
//makes two way enemy bounce
//--------------------------
int horzBounce(int theX,int theDir,int w)
{
	int start=enemy[w+1].type,end=enemy[w+3].type,setter=theDir;
	if(start<end)
		{
			if(theDir==-1 && theX==start)
			  setter=1;
	    if(theDir==1 && theX==end)
	      setter=2;
	  }
	if(start>end)
		{
			if(theDir==-1 && theX==end)
			  setter=1;
	    if(theDir==1 && theX==start)
	      setter=2;
	  }
	if(start==end) setter=0;
  enemy[w+7].type=setter;
  if(setter==2)
  	setter=-1;
  return setter;
}

//-------------------------
//makes flying enemy bounce
//-------------------------
int vertBounce(int theY,int theX,int theDir,int w)
{
	int start=enemy[w+2].type,end=enemy[w+4].type,setter=theDir;
	if(start<end)
		{
			if(theDir==2 && theY==start)
			  setter=1;
	    if(theDir==1 && theY==end)
	      setter=2;
	  }
	if(start>end)
		{
			if(theDir==2 && theY==end)
			  setter=1;
	    if(theDir==1 && theY==start)
	      setter=2;
	  }
	if(!(checkEDown(theX,theY)))
		setter=2;
	if(!(checkEUp(theX,theY)))
		setter=1;
	if(start==end) setter=0;
  enemy[w+8].type=setter;
  if(setter==2)
  	setter=-1;
  return setter;
}

void checkEHCollect(int cX,int cY,int d)
{
	if(d==1)
		d=-1;
	else
		d=8;
	if(map[cY/8][(cX+d)/8].type==1) 
		bolt(((cX+d)/8)*8,cY+add);
	if(map[cY/8][(cX+d)/8].type==2) 
		draw1up(((cX+d)/8)*8,cY+add);
}

void checkEVCollect(int cX,int cY,int v)
{
	if(v==1)
		v=-1;
	else
		v=8;
	if(map[(cY+v)/8][cX/8].type==1)
		bolt(cX,((cY+v)/8)*8+add);
	if(map[(cY+v)/8][cX/8].type==2)
		draw1up(cX,((cY+v)/8)*8+add);
}

//------------------
// draws all enemies
//------------------
void doEnemy(int w,int type)
{
	int xD=0,yD=0,eY,eX,d,v,a;
	eY=enemy[w+6].type;
	eX=enemy[w+5].type;
	d=enemy[w+7].type;
	v=enemy[w+8].type;
	a=enemy[w+9].type*160;				// compensates for screens after first one
	if(type!=3)
		yD=checkEDown(eX+a,eY); // collision detection below enemies 1 and 2
	if(v==0)
		checkEHCollect(eX+a,eY,d);			// redraws bolts and extra lives (horz)
	xD=checkHorz(eX+a,eY,d,w); // collision detection for horizontal
	if(type!=1)
		xD=horzBounce(eX,xD,w); // see if you bounce (horz) for enemy 2 and 3
	if(d==0 && type==3)
		{
			yD=vertBounce(eY,eX+a,v,w);  // checks if bounce for flying
			checkEVCollect(eX+a,eY,v);	// redraws bolts and extra lives (horz)
		}
	eX=eX+xD;
	masker(eX-xD,eY+add);	// erases enemy
	eY=eY+yD;
	if(type==1)
		oneDirEnemy(eX,eY+add);
	if(type==2)
	  twoDirEnemy(eX,eY+add);
	if(type==3)
	  flyingEnemy(eX,eY+add);
	borderCheck(eX,eY,w);
	enemy[w+5].type=eX;
	enemy[w+6].type=eY;
	for(time=0;time<timer;time++) {}  // waste some time
}

int enemies()
{
	int i;
	yes=0;
	for(i=0;i<(curEnemy*numE);i=i+numE)				// loops through all enemies
		{
			if(enemy[i+9].type==screen) // if on screen...
				{
					if(enemy[i].type!=0)    // and enemy is still in play...
					  {
							yes++;
							doEnemy(i,enemy[i].type);  // disply the enemy
						}
				}
		}
	if(yes>0)
		{
			for(time=0;time<timer;time++) {};  // if there were enemies, waste time
			return 1;
		}
	return 0;
}
